#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
A program to download my Lepidoptera data and generate a webpage!

Created on Fri Aug 16 16:23:40 2019
Version : 0.1 beta
@author: Bhalchandra Pujari 
@email : bspujari@cms.unipune.ac.in


http://api.inaturalist.org/v1/observations?user_id=bspujari&identified=true&parent_id=184884
"""

from halo import Halo #spinnner for loop
import requests
import json
import math
import time
import os.path
import collections
from collections import defaultdict
#import matplotlib.pyplot as plt
import pandas as pd
#from bokeh.io import output_file, show
from bokeh.palettes import Category20c
from bokeh.plotting import figure
from bokeh.transform import cumsum
from bokeh.embed import autoload_static
from bokeh.resources import CDN



class iNat_leps():
    
    def main(self):
        
        print ("Initializing ..")
        self.taxafile="taxa_id.dump" # a dump file to store taxa-id and names"
        self.name_rank_dict=defaultdict(list)
        self.TaxaId_Name={} # a grand dict storing all IDs and corresponding names. 
        
        if os.path.isfile(self.taxafile):
            with open(self.taxafile) as f:
                for line in f:
                    (key, val) = line.split(",")
                    self.TaxaId_Name[int(key)] = val
        
        
        
        print ('Downloading the data ... hang on..')
        self.iNat_data=[] # empty list to save data
        self.parse_iNat() # fetch the data from iNat and save in above list
        print ("Extracting the data..")
        self.extract_image_data() # extract the relevent data and 
        
        #for key in self.name_rank_dict:
        #create pie-charts
        print ("Plotting ..")
        self.plots()
        
        print ("Creating the webpage..")
        self.write_html("welcome.html")
        
        print ("Everything done. Please check 'welcome.html'. Good bye.")
        
        print (collections.Counter(self.name_rank_dict['family']))
        
    def plots(self): # make plots using bokeh
         self.htmlplots="" # some lines will be added in html. rest will go as javascript in dest dir
         plot_items=['genus','tribe','family','species']
         for item in plot_items :
             pie_data=collections.Counter(self.name_rank_dict[item])
             data=pd.Series(pie_data).reset_index(name='value').rename(columns={'index':item})
             data['angle'] = data['value']/data['value'].sum() * 2*math.pi
             #data['color'] = Category20c[len(pie_data)]
             p = figure(plot_height=350, title="Pie Chart", toolbar_location=None,
                        tools="hover", tooltips="@"+item+": @value")
             p.wedge(x=0, y=1, radius=0.4,
                     start_angle=cumsum('angle', include_zero=True), end_angle=cumsum('angle'),
                     line_color="white",# fill_color='color',
                    legend=item, source=data)
             js_dest="dist/js/bokeh_"+item+".js"
             js, tag = autoload_static(p, CDN, js_dest)
             f = open(js_dest,"w")
             f.write(js)
             f.close()
             self.htmlplots=self.htmlplots+tag+"\n"
             
             
#        plot_items=['genus','tribe','family','species']
#        for item in plot_items :
#            pie_data=collections.Counter(self.name_rank_dict[item])
#            labels=list(pie_data.keys())
#            values=list(pie_data.values())
#            plt.pie(values, labels=labels, shadow=True, startangle=140)
#            plt.savefig("inat"+item+".png")
#            plt.close()
        
    def write_html(self,fwrite):
        #copyfile("header.tx", fwrite)
        f= open(fwrite,"w")
        header=open("header.tx","r")
        footer=open("footer.tx","r")
        f.write(header.read())
        f.write(self.html_image_gallery)
        f.write(footer.read())
        
        f.close()
        footer.close()
        
        
        
    def extract_image_data(self):
        #count=0
        self.html_image_gallery="" # text to be appended in html file
        uniq_sci_name=[]
        spinner = Halo( text_color='green',spinner='dots')
        f=open(self.taxafile,'a')  # keep open the taxa_name file to append the unknonwn taxa_name pair
        for entry in self.iNat_data:
            spinner.start(text="Updating the database")
            for i in range(len(entry['results'])):
                if entry['results'][i]['place_guess'] and  "Ganeshkhind" in entry['results'][i]['place_guess']: # Extract  data only from within SPPU
                    ancestry=[int(z) for z in entry['results'][i]['taxon']["min_species_ancestry"].split(",")]
                    if 47157 in ancestry : # GET only Lepidoptera (taxa id=47157)
                        obs_id=entry['results'][i]['id'] 
                        obs_url="https://www.inaturalist.org/observations/"+str(obs_id)
                        sci_name=entry['results'][i]['taxon']['name'] 
                        rank=entry['results'][i]['taxon']['rank'] # it is a string corresponding to rank_level ('genus'/'family' etc)
                        rank_level=entry['results'][i]['taxon']['rank_level']
                        # ^^^^^^^ what is the ID rank? Is is species or genus or family?
                        taxa_id=entry['results'][i]['taxon']['min_species_taxon_id'] # taxa ID. 
                        
                        thumb_url=entry['results'][i]['observation_photos'][0]['photo']['url'].split('?')[0]
                        original_url=thumb_url.replace("square","original")
                        
                        if rank_level == 10 and sci_name not in uniq_sci_name: # gallery for uniq 'species'only.. but accept multiple for higher ranks
                            uniq_sci_name.append(sci_name)
                            #print (sci_name, rank,rank_level)
                            self.html_image_gallery = self.html_image_gallery +"<a href=\""+original_url+\
                               "\" data-lumenbox=\"my-gallery\"  data-title=\""+sci_name+" ("+rank+") Details:"+obs_url+"\"> <img src=\""+thumb_url+\
                                "\" alt=\""+sci_name+"\" /></a>\n"
                        else:
                            self.html_image_gallery = self.html_image_gallery +"<a href=\""+original_url+\
                               "\" data-lumenbox=\"my-gallery\"  data-title=\""+sci_name+" ("+rank+")\" > <img src=\""+thumb_url+\
                                "\" alt=\""+sci_name+"\" /></a>\n"
                                
                        #create a dict of ranks.. not uniq .. species/genus/family etc.. we'll plot this for abundance 
                        if taxa_id not in self.TaxaId_Name :
                            #print ("DEBUG", taxa_id) 
                            self.TaxaId_Name[taxa_id]= sci_name
                        # do a thourough search of taxa for name of all the ancesters (like tibe, family etc)
                        for ancester in ancestry :
                            if ancester not in self.TaxaId_Name :
                                #print(obs_id,ancester)
                                api_string="http://api.inaturalist.org/v1/taxa/"+str(ancester)
                                r=requests.get(api_string) # download .. make sure proxies are fine
                                r=json.loads(r.text)
                                taxaname=r['results'][0]['name']
                                self.TaxaId_Name[int(ancester)]= taxaname
                                f.write(str(ancester)+","+str(taxaname)+"\n")
                                #
                                #print(ancester,r['results'][0]['name'])
                                
                            
                        #if sci_name not in self.name_rank_dict[rank]:
                        self.name_rank_dict[rank].append(sci_name)
                        
        f.close()
        spinner.succeed("Extraction done")    
        #print (self.TaxaId_Name)
            
            

    def parse_iNat(self): 
      
      page_count=1 # Required by the format of API 
           # total observations = pages * observations per page
           #                  (set to 100 at this stage)
      total_pages='NA'
      
      
      
      spinner = Halo( text_color='green',spinner='dots')
      t0=time.time()
      while True:
      #while page_count <2:
          
          if page_count == 1 : 
              spin_text="Page "+str(page_count)
          else :
              spin_text="Page "+str(page_count)+" of "+ total_pages+\
              "  Time Remaining : " + str(math.ceil((t1-t0)*(int(total_pages)-page_count+1))) + " sec"
          spinner.start(text=spin_text )
          

              # Build the string for API download
          api_string="http://api.inaturalist.org/v1/observations?"+\
          "user_id=bspujari"+\
          "&identified=true"+\
          "&photos=true"+\
          "&page="+str(page_count)+\
          "&per_page=100"+\
          "&order=desc"+\
          "&order_by=created_at"
          # looks like 100 is the limit for per page observations.
          
          
          r=requests.get(api_string) # download .. make sure proxies are fine
          j=json.loads(r.text)
          
          
          if page_count == 1 : # for the first iteration, determine how many pages of downloadng is there..
              total_pages=str(math.ceil(j['total_results']/100) +1) # total results/results per page
              t1=time.time()
              
          if j['results'] : # error will occur when page numbers are exhausted.. till that time keep increasing page numbers.
              page_count+=1
              self.iNat_data.append(json.loads(r.text))
          else:
              break
          
          
          spinner.stop()
      spinner.succeed("Download complete") 
if __name__ == '__main__':
   u=iNat_leps()
   u.main()
